@@ -374,6 +374,86 @@ class Sequence(Generic[T], Iterable[T]):
         for i in self if predicate is None else self.filter(predicate):
             last = i
         return last
+    
+    def index_of(self, element: T) -> int:
+        """
+         Returns first index of [element], or -1 if the collection does not contain element.
+         
+         Example 1:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).index_of('b')
+        1
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).index_of('d')
+        -1
+        """
+        for i, x in enumerate(self._iter):
+            if x == element:
+                return i
+        return -1
+    
+    def last_index_of(self, element: T) -> int:
+        """
+         Returns last index of [element], or -1 if the collection does not contain element.
+         
+         Example 1:
+        >>> lst = ['a', 'b', 'c', 'b']
+        >>> it(lst).last_index_of('b')
+        3
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).last_index_of('d')
+        -1
+        """
+        seq = self.reversed()
+        last_idx = len(seq) - 1;
+        for i, x in enumerate(seq):
+            if x == element:
+                return last_idx - i
+        return -1
+    
+    def index_of_first(self, predicate: Callable[[T], bool]) -> int:
+        """
+         Returns first index of element matching the given [predicate], or -1 if no such element was found.
+         
+         Example 1:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).index_of_first(lambda x: x == 'b')
+        1
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).index_of_first(lambda x: x == 'd')
+        -1
+        """
+        for i, x in enumerate(self._iter):
+            if predicate(x):
+                return i
+        return -1
+    
+    def index_of_last(self, predicate: Callable[[T], bool]) -> int:
+        """
+         Returns last index of element matching the given [predicate], or -1 if no such element was found.
+         
+         Example 1:
+        >>> lst = ['a', 'b', 'c', 'b']
+        >>> it(lst).index_of_last(lambda x: x == 'b')
+        3
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).index_of_last(lambda x: x == 'd')
+        -1
+        """
+        seq = self.reversed()
+        last_idx = len(seq) - 1;
+        for i, x in enumerate(seq):
+            if predicate(x):
+                return last_idx - i
+        return -1
 
     @overload
     def single(self) -> T:
@@ -1194,8 +1274,8 @@ class Sequence(Generic[T], Iterable[T]):
         """
         return FlatteningSequence(self, transform)
     
-
-    def flatten(self) -> Sequence[T]:
+    @overload
+    def flatten(self: Sequence[List[R]]) -> Sequence[R]:
         """
          Returns a sequence of all elements from all sequences in this sequence.
 
@@ -1204,7 +1284,26 @@ class Sequence(Generic[T], Iterable[T]):
         >>> it(lst).flatten()
         ['a', 'b', 'c', 'd', 'e']
         """
-        return FlatteningSequence(self, lambda x: x)
+        ...
+    @overload
+    def flatten(self: Sequence[Set[R]]) -> Sequence[R]:
+        ...
+    @overload
+    def flatten(self: Sequence[Tuple[R, ...]]) -> Sequence[R]:
+        ...
+    @overload
+    def flatten(self: Sequence[Sequence[R]]) -> Sequence[R]:
+        ...
+    def flatten(self: Sequence[Iterable[R]]) -> Sequence[R]:
+        """
+         Returns a sequence of all elements from all sequences in this sequence.
+
+         Example 1:
+        >>> lst = [['a', 'b'], ['c'], ['d', 'e']]
+        >>> it(lst).flatten()
+        ['a', 'b', 'c', 'd', 'e']
+        """
+        return FlatteningSequence(self)
     
 
     def group_by(self, key_selector: Callable[[T], K]) -> GroupingSequence[K, T]:
@@ -1712,7 +1811,7 @@ class ParallelMappingSequence(Sequence[R]):
             transformer: Callable[[T], R], 
             max_workers: Optional[int]=None, 
             chunksize: int=1,
-            executor: Executor = 'Thread') -> None:
+            executor: ParallelMappingSequence.Executor = 'Thread') -> None:
 
         super().__init__(iterable)
         self._transformer = transformer
@@ -1769,13 +1868,17 @@ class IndexingSequence(Sequence[IndexedValue[T]]):
 
 
 class FlatteningSequence(Sequence[R]):
-    def __init__(self, iterable: Iterable[Sequence[T]], transformer: Callable[[T], R]) -> None:
+    def __init__(self, iterable: Iterable[Sequence[T]], transformer: Optional[Callable[[T], R]]=None) -> None:
         super().__init__(iterable)
         self._transformer = transformer
     
     def __do_iter__(self) -> Iterator[R]:
-        for i in self._iter:
-            yield from self._transformer(i)
+        if self._transformer:
+            for i in self._iter:
+                yield from self._transformer(i)
+        else:
+            for i in self._iter:
+                yield from i
 
 
 class DropSequence(Sequence[T]):
